<?php
/*
 * Copyright © 2021 ExacTI Technology Solutions. All rights reserved.
 * GPLv3 General License.
 * https://exacti.com.br
 * Phacil PHP Framework - https://github.com/exacti/phacil-framework
 */

namespace Phacil\Framework;

/** @package Phacil\Framework */
class Config {

	/**
	 * @var \Phacil\Framework\startEngineExacTI
	 */
	private $engine;

	/**
	 * @var array
	 */
	private $data = array();

	public function __construct(
		\Phacil\Framework\startEngineExacTI $engine
	) {
		$this->engine = $engine;
	}

	/**
	 * @return void 
	 * @throws \Phacil\Framework\Exception 
	 */
	public function initialize() {
		if (\Phacil\Framework\Config::USE_DB_CONFIG() === true) {
			if(!$this->engine->db->query()->isTableExists('settings')){
				$erro = new \Phacil\Framework\Exception\NotFoundException(sprintf("The table '%s' does not exist", 'settings'), 3);
				return;
			}
			$query = $this->engine->db->query()->select()->from('settings')->orderBy('setting_id', \Phacil\Framework\MagiQL\Api\Syntax\OrderBy::ASC)->load();

			if($query instanceof \Phacil\Framework\Databases\Api\Object\ResultInterface){
				foreach ($query as $setting) {
					if (!$setting['serialized']) {
						$this->set($setting['key'], $setting['value']);
					} else {
						$this->set($setting['key'], unserialize($setting['value']));
					}
				}
			}
		}
	}

  	/**
  	 * @param string|int $key 
  	 * @return null|array|string 
  	 */
  	public function get($key) {
    	return ($this->has($key) ? $this->data[$key] : null);
	}
	
	/**
	 * @param string $key 
	 * @param mixed $value 
	 * @return void 
	 */
	public function set($key, $value) {
    	$this->data[$key] = $value;
  	}

	/**
	 * @param string|int $key 
	 * @return bool 
	 */
	public function has($key) {
    	return isset($this->data[$key]);
  	}

	/**
	 * 
	 * @param string $key 
	 * @param mixed $value 
	 * @return mixed 
	 */
	static public function __callStatic ($key, $value){
		try {
			return (defined($key)) ? constant($key) : ($value ? self::setConstant($key, $value) : null);
		} catch (\Phacil\Framework\Exception $th) {
			throw new \Phacil\Framework\Exception($th->getMessage());
		}
	}

	/**
	 * 
	 * @param string $key 
	 * @param mixed $value 
	 * @return mixed 
	 */
	static private function setConstant($key, $value){
		if(is_array($value) && count($value) == 1){
			$value = end($value);
		}
		$php = version_compare(phpversion(), '7.0', '>=');
		if(is_array($value) && !$php){
			throw new \Phacil\Framework\Exception('Array is not supported on constant value in PHP <7');
		}
		return (define($key, $value)) ? $value : null;
	}

  	/**
  	 * @param string $filename 
  	 * @return void 
  	 */
  	public function load($filename) {
		$file = self::DIR_CONFIG() . $filename . '.php';
		
    	if (file_exists($file)) { 
	  		$cfg = array();
	  
	  		require($file);
	  
	  		$this->data = array_merge($this->data, $cfg);
		} else {
			throw new \Phacil\Framework\Exception('Could not load config ' . $filename. '!');
		}
  	}
}
